
local COLOR_LABEL = "|cffffcc00"
local COLOR_URL = "|cffaaaaff"

--
-- Creates an option.
--
-- @param theOrder the order in which this option should appear.
-- @param value the value to display.
-- @return a table describing the option, or nil if the value is nil.
--
local function _CreateOption( theOrder, value )
	if not value then
		return nil
	end

	return {
		type = "description",
		order = theOrder,
		width = "full",
		fontSize = "medium",
		name = value,
	}
end

--
-- Creates a labeled option.
-- 
-- @param theOrder the order in which this option should appear.
-- @param label a label to prepend the option's value.
-- @param value the value to display.
-- @return a table describing the option, or nil if the value is nil.
--
local function _CreateLabeledOption( order, label, value )
	if not value then
		return nil
	end
	
	if label then
		value = COLOR_LABEL .. label .. ":|r " .. value
	end

	return _CreateOption( order, value )
end

--
-- Creates a spacer option to put vertical white space between 
-- options.
-- 
-- @param order the order in which this option should appear.
-- @param lines the number of lines of whitespace.
-- @param a table decsribing the option, or nil if the number of lines 
-- is less than one.
--
local function _CreateSpacerOption( order, lines )
	if not lines then
		lines = 1
	end
	
	if lines < 1 then
		return nil
	end
	
	return _CreateOption( order, strrep( "\n", lines ))
end

--
-- Creates an author option from a set of metadata fields. This gets 
-- the author name from "Author", and optionally includes realm and 
-- faction names from "X-Author-Server", "X-Author-Realm", and 
-- "X-Author-Faction".
--
-- @param order the order in which this option should appear.
-- @param name the name of an addon.
-- @return a table describing the option, or nil if the author is 
-- undefined.
--
local function _CreateAuthorOption( order, name )
	local author = GetAddOnMetadata( name, "Author" )
	if not author then
		return nil
	end
	
	local realm = GetAddOnMetadata( name, "X-Author-Server" ) or GetAddOnMetadata( name, "X-Author-Realm" )
	if realm then
		author = author .. " on " .. realm
	end
	
	local faction = GetAddOnMetadata( name, "X-Author-Faction" )
	if faction then
		author = author .. " (" .. faction .. ")"
	end
	
	return _CreateLabeledOption( order, "Author", author)
end

--
-- Creates a generic option from a metadata field.
--
-- @param order the order in which this option should appear.
-- @param label an optional label which will prepend the option's value.
-- @param name the name of an addon.
-- @param field the name of a metadata field in the addon's .TOC file.
-- @param order the order to assign to the resulting option.
-- @return a table describing the option, or nil if the field is 
-- undefined.
--
local function _CreateGenericOption( order, label, name, field )
	return _CreateLabeledOption( order, label, GetAddOnMetadata( name, field ) )
end

--
-- Creates a URL option from a metadata field.
--
-- @param order the order in which this option should appear.
-- @param label an optional label which will prepend the option's value.
-- @param name the name of an addon.
-- @param field the name of a metadata field in the addon's .TOC file.
-- @param order the order to assign to the resulting option.
-- @return a table describing the option, or nil if the field is 
-- undefined.
--
local function _CreateUrlOption( order, label, name, field )
	local value = GetAddOnMetadata( name, field )
	if not value then
		return nil
	else
		return _CreateLabeledOption( order, label, COLOR_URL .. value )
	end
end

--
-- Creates a set of about options from a given addon's metadata.
--
-- @param name the name of an addon.
--
local function _CreateAboutOptions( name )
	return {
		type = "group",
		name = "About " .. name,
		args = {
			notes = _CreateGenericOption( 10, nil, name, "Notes" ),
			spacer1 = _CreateSpacerOption( 19, 1 ),
			author = _CreateAuthorOption( 20, name ),
			email = _CreateUrlOption( 21, "E-Mail", name, "X-EMail" ),
			website = _CreateUrlOption( 22, "Website", name, "X-Website" ),
			spacer2 = _CreateSpacerOption( 29, 1 ),
			version = _CreateGenericOption( 30, "Version", name, "Version" ),
			category = _CreateGenericOption( 31, "Category", name, "X-Category" ),
			license = _CreateGenericOption( 32, "License", name, "X-License" ),
		},
	}
end

--
-- Creates an about panel in the interface options window using the 
-- metadata from the given addon's .TOC file.
--
-- @param name the name of the addon for which the about panel should 
-- be created.
-- @return the option frame.
--
function CreateAboutOptionFrame( name )
	LibStub( "AceConfig-3.0" ):RegisterOptionsTable( name .. "About", _CreateAboutOptions( name ) )
	return LibStub( "AceConfigDialog-3.0" ):AddToBlizOptions( name .. "About", "About", name )
end
